
/* Portions Copyright (c) 1995,1996,1997 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




/*
 * Program to translate base64 encoding into something more readable
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

static unsigned char rev_table[256] = {
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,   62,  255,  255,  255,   63,
          52,   53,   54,   55,   56,   57,   58,   59,
          60,   61,  255,  255,  255,  255,  255,  255,
         255,    0,    1,    2,    3,    4,    5,    6,
           7,    8,    9,   10,   11,   12,   13,   14,
          15,   16,   17,   18,   19,   20,   21,   22,
          23,   24,   25,  255,  255,  255,  255,  255,
         255,   26,   27,   28,   29,   30,   31,   32,
          33,   34,   35,   36,   37,   38,   39,   40,
          41,   42,   43,   44,   45,   46,   47,   48,
          49,   50,   51,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255};

#define octet_one(p)	(((p)[0] << 2) | (((p)[1] >> 4) & 0x3))
#define octet_two(p)	((((p)[1] << 4) & 0xf0) | (((p)[2] >> 2) & 0xf))
#define octet_three(p)	(((((p)[2]) & 0x3) << 6) | ((p)[3]))

decode64( bin, bin_len, bout)
	char           *bin;
	int             bin_len;
        unsigned char   bout[];
{
	int             length, si, di, bits,i;
	unsigned char   buf[4];

	/* DOUBLE CHECK PARAMETERS */
	if (bin == NULL)  return (-1);

	/* CHECK FOR ZERO LENGTH INPUT BUFFER */
	if (bin_len == 0) return (0);

	/* CHECK THAT INPUT BUFFER LENGTH IS MULTIPLE OF 4 */
	if (bin_len % 4 != 0) return (-2);

	/* ALLOCATE OUTPUT BUFFER OF APPROPRIATE LENGTH */
	length = (bin_len * 3) / 4;
	if (bin[bin_len - 1] == '=') 	length--;
	if (bin[bin_len - 2] == '=')	length--;

	/* DECODE ALL BUT LAST 4 BYTES */
	for (di = si = 0; si < bin_len - 4; si += 4, di += 3) {
		buf[0] = rev_table[bin[si]];
		buf[1] = rev_table[bin[si + 1]];
		buf[2] = rev_table[bin[si + 2]];
		buf[3] = rev_table[bin[si + 3]];
		bout[di] = octet_one(buf);
		bout[di + 1] = octet_two(buf);
		bout[di + 2] = octet_three(buf);
	}

	/* DECODE THE LAST 4 BYTES */
	buf[0] = rev_table[bin[si]];
	buf[1] = rev_table[bin[si + 1]];
	buf[2] = rev_table[bin[si + 2]];
	buf[3] = rev_table[bin[si + 3]];
	bout[di] = octet_one(buf);
	if (bin[si + 2] != '=') {
		bout[di + 1] = octet_two(buf);
		if (bin[si + 3] != '=') {
			bout[di + 2] = octet_three(buf);
		}
	}
	bits = length*8;
/* this loop looks for the most significant bit that is turned on */
	for( i =0x80; (i > 0x0) && ( (*bout&i) == 0x0 ); i>>=1)
	  bits--;
	return bits;
}




main(argc, argv )
char **argv;
int argc;
{
  char *cp,in_string[10000], out_string[10000];
  int len=0, hex = 0, olen=0,i;

  if( argc >=2  && argv[1][1] == 'x') hex =1;

  while( fgets( in_string, sizeof(in_string), stdin)){
    i = strlen(in_string);
    if( i > 7 && ((strncmp(in_string, "dnssec_sign",11) ==0) ||
		  (strncmp(in_string, "dnssec_veri",11) ==0) ||
		  (strncmp(in_string, "dnssec():",   9) ==0))) { 
      cp = strchr( in_string, ' '); /* get the first space */
      while( isspace(*cp)) cp++;
      len = atoi(cp);
      cp = strchr(cp, ' ');
      while( isspace(*cp)) cp++;
      /* at the start of the buffer */
      olen = decode64(cp, len, out_string)/8+1;
    }
    else if( i > 80) {  /* guess for signature or KEY */
      for( len=i, cp = in_string; isspace(*cp); cp++)len--;
      olen = decode64(cp, --len, out_string)/8+1;
    }

    for( i=0; i < olen; i++) {
      if( hex ) printf(" %2x ", (u_char) out_string[i]);
      else if( out_string[i] & 0x80) printf(" %2x ", (u_char) out_string[i]);
      else if( isprint(out_string[i])) printf("%c",out_string[i]);
      else printf(" %2x ", (u_char) out_string[i]);
      if( (i % 20)==19) printf("\n");
    }
    printf("\n%d\n",len);
    memset(in_string, 0, len+30);
    olen = len =0;
  } /* while */
}
  
